from onegov.core.orm import ModelBase
from onegov.core.orm import SessionManager
from onegov.core.orm.mixins import ContentMixin
from onegov.gis.models import Coordinates, CoordinatesMixin
from sqlalchemy import Column
from sqlalchemy import Integer
from sqlalchemy.ext.declarative import declarative_base


def test_mutable_coordinates(postgres_dsn):
    Base = declarative_base(cls=ModelBase)

    class Monument(Base, ContentMixin, CoordinatesMixin):
        __tablename__ = 'monument'
        id = Column(Integer, primary_key=True)

    mgr = SessionManager(postgres_dsn, Base)
    mgr.set_current_schema('foo')

    session = mgr.session()
    session.add(Monument())
    session.flush()

    washington = session.query(Monument).one()
    assert washington.coordinates.lat is None
    assert washington.coordinates.lon is None
    assert washington.coordinates.zoom is None

    washington = session.query(Monument).one()
    washington.coordinates = Coordinates(38.889484, -77.035278)
    assert washington.coordinates.lat == 38.889484
    assert washington.coordinates.lon == -77.035278
    assert washington.coordinates.zoom is None
    session.flush()

    washington = session.query(Monument).one()
    washington.coordinates.zoom = 10
    session.flush()

    assert session.query(Monument).one().coordinates.zoom == 10

    mgr.dispose()



import morepath
import pytest

from onegov.core import Framework
from onegov.core import filters  # noqa -> registers webasset filters
from onegov.core.utils import scan_morepath_modules
from onegov.gis import MapboxApp
from webtest import TestApp as Client


def test_no_secret_keys(temporary_directory):

    class App(Framework, MapboxApp):
        pass

    @App.webasset_output()
    def get_output_path():
        return temporary_directory

    morepath.commit(App)
    app = App()

    with pytest.raises(AssertionError):
        app.configure_application(mapbox_token='sk.asdf')


def test_mapbox_token_tween(temporary_directory):

    class App(Framework, MapboxApp):
        pass

    @App.webasset_output()
    def get_output_path():
        return temporary_directory

    @App.path(path='')
    class Root(object):
        pass

    @App.html(model=Root)
    def view_root(self, request):
        return '<body></body>'

    scan_morepath_modules(App)
    morepath.commit(App)

    app = App()
    app.configure_application(mapbox_token='pk.asdf')
    app.namespace = 'foo'
    app.set_application_id('foo/bar')

    assert '<body data-mapbox-token="pk.asdf"></body>' in Client(app).get('/')



from onegov.core.custom import json


class Coordinates(json.Serializable, keys=('lon', 'lat', 'zoom')):
    """ Represents a pair of coordinates.

    May contain zoom factor and other information like marker icon and
    color. Note that only latitude and longitude really matter, the rest
    may or may not be used.

    """

    __slots__ = ('lon', 'lat', 'zoom')

    def __init__(self, lat=None, lon=None, zoom=None):
        self.lat = lat
        self.lon = lon
        self.zoom = zoom

    def __bool__(self):
        return False if None in (self.lat, self.lon) else True

    def __eq__(self, other):
        if not isinstance(other, Coordinates):
            return False

        return self.lat == other.lat and\
            self.lon == other.lon and\
            self.zoom == other.zoom


class CoordinatesMixin(object):
    """ Extends any class that has a content dictionary field with a single
    coordinates pair.

    """

    @property
    def coordinates(self):
        return self.content.get('coordinates') or Coordinates()

    @coordinates.setter
    def coordinates(self, value):
        self.content['coordinates'] = value or {}



from wtforms.widgets import TextInput


class CoordinatesWidget(TextInput):
    """ Widget holding and showing the data behind the
    :class:`onegov.gis.forms.fields.CoordinatesField` class.

    Basically a textfield that stores json. Meant to be enhanced on the browser
    using javascript.

    """

    def __call__(self, field, **kwargs):
        kwargs['class_'] = (kwargs.get('class_', '') + ' coordinates').strip()
        return super().__call__(field, **kwargs)



import logging
log = logging.getLogger('onegov.gis')  # noqa
log.addHandler(logging.NullHandler())  # noqa

from onegov.gis.forms import CoordinatesField
from onegov.gis.integration import MapboxApp
from onegov.gis.models import Coordinates, CoordinatesMixin

__all__ = ['Coordinates', 'CoordinatesMixin', 'CoordinatesField', 'MapboxApp']



import re

from base64 import b64encode, b64decode
from onegov.core.custom import json
from onegov.form import Form
from onegov.gis.models import Coordinates
from onegov.gis.forms import CoordinatesField


def test_coordinates_field():
    value = re.compile(r'value="([a-zA-Z0-9=]+)"')

    # initially the field contains empty coordinates
    field = CoordinatesField().bind(Form(), 'coordinates')
    assert not field.data
    assert field.data.lat is None
    assert field.data.lon is None
    assert field.data.zoom is None

    # this holds true for the rendered field
    coordinate = json.loads(b64decode(value.search(field()).group(1)))

    assert coordinate.lat is None
    assert coordinate.lon is None
    assert coordinate.zoom is None

    # when we process a dictionary we get non-empty coordinates
    field.process_formdata([b64encode(
        json.dumps({
            'lat': 47.05183585,
            'lon': 8.30576869173879,
            'zoom': 10
        }).encode('ascii')
    ).decode('ascii')])

    assert field.data.lat == 47.05183585
    assert field.data.lon == 8.30576869173879
    assert field.data.zoom == 10

    # which again holds true for the rendered field
    coordinate = json.loads(b64decode(value.search(field()).group(1)))

    assert coordinate.lat == 47.05183585
    assert coordinate.lon == 8.30576869173879
    assert coordinate.zoom is 10

    # when we process a dictionary coordinates are created
    field.process_data({
        'lat': 47.05183585,
        'lon': 8.30576869173879,
        'zoom': None
    })

    assert field.data.lat == 47.05183585
    assert field.data.lon == 8.30576869173879
    assert field.data.zoom is None

    # on an object, coordinates are stored as is
    class Test(object):
        pass

    test = Test()
    field.populate_obj(test, 'coordinates')

    assert test.coordinates.lat == 47.05183585
    assert test.coordinates.lon == 8.30576869173879

    field.data = Coordinates()
    field.populate_obj(test, 'coordinates')
    assert test.coordinates.lat is None
    assert test.coordinates.lon is None
    assert test.coordinates.zoom is None
